unit DIntersection;

interface

uses
  DMath;

function Contain(A, B: TRect): Boolean;
function Intersect(A, B: TRect): Boolean; overload;
function Intersect(R: TRect; X1, X2, Y: Single): Boolean; overload;
// Delta -    R,  R  A  
function Intersect(R: TRect; A: TVec2f; var Delta: TVec2f): Boolean; overload;
function Intersect(R: TRect; A: TVec2f): Boolean; overload;

function Intersect(Line: TLine2f; Rect: TRect; 
                   var T: Single): Boolean; overload;
function Intersect(Pos, Dir: TVec2f; Rect: TRect; 
                   var T: Single): Boolean; overload;

implementation

{$REGION 'functions'}
function Contain(A, B: TRect): Boolean;
begin
  Contain := (A.A.X <= B.A.X) and (B.B.X <= A.B.X) and
             (A.A.X <= B.A.X) and (B.B.X <= A.B.X);
end;

function Intersect1D(A, B, C, D: Single): Boolean;
begin
  Intersect1D := ((A <= C)and(C <= B)) or ((A <= D)and(D <= B)) or
                 ((C <= A)and(A <= D)) or ((C <= B)and(B <= D));
end;

function Intersect(A, B: TRect): Boolean; overload;
begin
  Intersect := Intersect1D(A.A.X, A.B.X, B.A.X, B.B.X) and
               Intersect1D(A.A.Y, A.B.Y, B.A.Y, B.B.Y);
end;

function Intersect(R: TRect; X1, X2, Y: Single): Boolean;
begin
  Intersect := Intersect(R, Rect(Vec2f(X1, Y), Vec2f(X2, Y)));
end;

function Intersect(R: TRect; A: TVec2f; var Delta: TVec2f): Boolean;
begin
  if A.X > R.B.X then
    Delta.X := A.X - R.B.X
  else if A.X < R.A.X then
    Delta.X := A.X - R.A.X
  else
    Delta.X := 0.0;
  if A.Y > R.B.Y then
    Delta.Y := A.Y - R.B.Y
  else if A.Y < R.A.Y then
    Delta.Y := A.Y - R.A.Y
  else
    Delta.Y := 0.0;
  Intersect := (Delta.X = 0.0) and (Delta.Y = 0.0);
end;

function Intersect(R: TRect; A: TVec2f): Boolean;
begin
  Intersect := (R.A.X <= A.X) and (A.X <= R.B.X) and
               (R.A.Y <= A.Y) and (A.Y <= R.B.Y)
end;

function Intersect(Line: TLine2f; Rect: TRect; var T: Single): Boolean;
begin
end;

function Intersect(Pos, Dir: TVec2f; Rect: TRect; var T: Single): Boolean;
  var
    Line: TLine2f;
    V: array[0..3] of record
                     P: TVec2f;
                     D: Single;
                   end;
    I: Integer;
    Temp, A, B: Single;
    P: TVec2f;
begin
  Intersect := False;
  T := 10e+10;
  V[0].P := Rect.A;
  V[1].P := Vec2f(Rect.B.X, Rect.A.Y);
  V[2].P := Rect.B;
  V[3].P := Vec2f(Rect.A.X, Rect.B.Y);
  Line := DMath.Line2f(Pos, Pos + Dir);
  Dir := Dir / Len2f(Dir);
  for I := 0 to 3 do
    V[I].D := Dist2f(Line, V[I].P);
  for I := 0 to 3 do 
    if V[I].D * V[(I+1) mod 4].D < 0 then begin
      A := Abs(V[I].D);
      B := Abs(V[(I+1) mod 4].D);
      P := (B * V[I].P  +  A * V[(I + 1) mod 4].P) / (A + B);
      Temp := Dot2f(Dir, P - Pos);
      if (0 < Temp) and (Temp < T) then begin
        T := Temp;
        Intersect := True;
      end;
    end;
end;
{$ENDREGION}

end.
