{
         -   TGui  TWidget.

         TGui
          *    - , winapi-  ..
          *      ... (!)
}
unit DGuiCore;

interface

uses
  DDebug,
  DClasses,
  DMath,
  DIntersection,
  DEvents,
  DUtils,
  DWinApi,
  DGraph,
  DFont;

type
TGuiStyle = class;
TGui = class;
TWidget = class;

// 
P = Integer;

TGuiResource = class
private
  FStyle: TGuiStyle;
  FName: String;
public
//  constructor Create(Style: TGuiStyle; Name: String);
  property Name: String read FName;
end;

TGuiFont = class(TFont)
private
  FStyle: TGuiStyle;
  FName: String;
public
  property Style: TGuiStyle read FStyle write FStyle;
  property Name: String read FName write FName;
end;
TListOfGuiFont = {$IFDEF FLAG_FPC}specialize{$ENDIF} TList<TGuiFont>;

TGuiImage = Integer;

TListOfWidget = {$IFDEF FLAG_FPC}specialize{$ENDIF} TList<TWidget>;

TProcOfObj = procedure of object;
TOnClick = procedure (const Button: TMouseButton; const Pos: TVec2f) of object;
TOnRender = procedure of object;

TGuiStyle = class
private
  FGui: TGui;
  FName: String;
  FPrimaryColor: TVec4f;
  FSecondaryColor: TVec4f;
  FBackColor: TVec4f;
  FDisabledColor: TVec4f;
  FFonts: TListOfGuiFont;
public
  constructor Create(Gui: TGui; Name: String);
  procedure Color4f(Color: TVec4f); virtual;
  procedure DrawRect(Rect: TRect; Fill: Boolean = True); virtual;
  procedure DrawCircle(Center: TVec2f; Radius: Single; Fill: Boolean = True); virtual;
  procedure SetLineWidth(Width: Single); virtual;
  procedure DrawLine(A, B: TVec2f); virtual;
  procedure DrawImage(Rect: TRect; Image: TGuiImage); virtual;
  procedure DrawText(Pos, Align: TVec2f; Font: TGuiFont; Text: string); virtual;
  procedure AddFont(Name, FileName: String; Spacing: Integer = 0);
  function FindFont(Name: String): TGuiFont;
  property Gui: TGui read FGui;
  property Name: String read FName;
  property PrimaryColor: TVec4f read FPrimaryColor write FPrimaryColor;
  property SecondaryColor: TVec4f read FSecondaryColor write FSecondaryColor;
  property BackColor: TVec4f read FBackColor write FBackColor;
  property DisabledColor: TVec4f read FDisabledColor write FDisabledColor;
end;
TListOfGuiStyle = {$IFDEF FLAG_FPC}specialize{$ENDIF} TList<TGuiStyle>;

TGui = class(TEntity)
private
  FGraph: TGraph;
  FWindow: TWindow;
  FFontMan: TFontMan;
  FRoot: TWidget;
  FFocus: TWidget;
  FStyles: TListOfGuiStyle;
  function GetMouse: TVec2f;
  function RecOnMouseDown(Widget: TWidget; Button: TMouseButton;
                          Pos: TVec2f; Flags: Cardinal): Boolean;
public
  constructor Create(Graph: TGraph; Window: TWindow);
  destructor Destroy; override;
  procedure Render; override;
  procedure Update; override;
  procedure OnEvent(Event: TEvent); override;
  procedure OnMouseDown(Button: TMouseButton; Pos: TVec2f; Flags: Cardinal);
  procedure AddStyle(Style: TGuiStyle);
  function FindStyle(Name: String): TGuiStyle;
  procedure Link(Parent, Widget: TWidget);
  procedure Unlink(Widget: TWidget);
  procedure Popup(Widget: TWidget);
  function Trace(Point: TVec2f): TWidget; //     
  procedure SetFocus(Widget: TWidget);
  property Graph: TGraph read FGraph;
  property FontMan: TFontMan read FFontMan write FFontMan;
  property Root: TWidget read FRoot;
  property Focus: TWidget read FFocus write SetFocus;
  property Styles[Index: String]: TGuiStyle read FindStyle;
  property Mouse: TVec2f read GetMouse;
end;

TWidget = class(TEntity)
private
  FGui: TGui;
  FName: String;
  FStyle: TGuiStyle;
  FParent: TWidget;
  FSub: TListOfWidget;
  FShow: Boolean;
  FLocal: TVec2f;
  FSize: TVec2f;
  FFocusable: Boolean;
  function GetGlobal: TVec2f;
  procedure SetGlobal(NewPos: TVec2f);
  function GetLocalCenter: TVec2f;
  procedure SetLocalCenter(NewCenter: TVec2f);
  function GetGlobalCenter: TVec2f;
  procedure SetGlobalCenter(NewCenter: TVec2f);
  function GetLocalRect: TRect;
  procedure SetLocalRect(NewRect: TRect);
  function GetGlobalRect: TRect;
  procedure SetGlobalRect(NewRect: TRect);
public
  constructor Create(Gui: TGui; Name, Style: String);
  destructor Destroy; override;
  procedure SetParent(Parent: TWidget);
  function IsLocalPointIn(Point: TVec2f): Boolean; inline;
  function IsGlobalPointIn(Point: TVec2f): Boolean; inline;
  function Trace(Point: TVec2f): TWidget;
  function IsMouseIn: Boolean;
  function OnMouseDown(Button: TMouseButton; Pos: TVec2f; 
                       Flags: Cardinal): Boolean; virtual;
  procedure OnRender;
  procedure OnUpdate;
  procedure SetFocus;
  procedure Render; override;
  procedure Update; override;
  property Gui: TGui read FGui;
  property Name: String read FName;
  property Style: TGuiStyle read FStyle;
  property Parent: TWidget read FParent write SetParent;
  property Sub: TListOfWidget read FSub;
  property Show: Boolean read FShow write FShow;
  property Local: TVec2f read FLocal write FLocal; 
  property Global: TVec2f read GetGlobal write SetGlobal;
  property LocalCenter: TVec2f read GetLocalCenter write SetLocalCenter;
  property GlobalCenter: TVec2f read GetGlobalCenter write SetGlobalCenter;
  property Size: TVec2f read FSize write FSize;
  property LocalRect: TRect read GetLocalRect write SetLocalRect;
  property GlobalRect: TRect read GetGlobalRect write SetGlobalRect;
  property Focusable: Boolean read FFocusable;
end;

TWidgetDecorator = class
private
  FWidget: TWidget;
  function GetGui: TGui;
  function GetStyle: TGuiStyle;
public
  constructor Create(Widget: TWidget);
  property Widget: TWidget read FWidget;
  property Gui: TGui read GetGui;
  property Style: TGuiStyle read GetStyle;
end;

implementation

{$REGION 'TGuiStyle'}
constructor TGuiStyle.Create(Gui: TGui; Name: String);
begin
  FGui := Gui;
  FName := Name;
  FFonts := TListOfGuiFont.Create;
end;

procedure TGuiStyle.Color4f(Color: TVec4f);
begin
  Gui.Graph.Color4f(Color);
end;

procedure TGuiStyle.DrawRect; 
begin
  if Fill then
    Gui.Graph.DrawRect(Rect)
  else
    Gui.Graph.DrawRect(Rect, False);
end;

procedure TGuiStyle.DrawCircle(Center: TVec2f; Radius: Single; Fill: Boolean = True);
begin
  Gui.Graph.DrawCircle(Center, Radius, Fill);
end;

procedure TGuiStyle.SetLineWidth(Width: Single);
begin
  Gui.Graph.SetLineWidth(Width);
end;

procedure TGuiStyle.DrawLine(A, B: TVec2f);
begin
  Gui.Graph.DrawLine(A, B);
end;

procedure TGuiStyle.DrawImage(Rect: TRect; Image: TGuiImage);
begin
end;

procedure TGuiStyle.DrawText(Pos, Align: TVec2f; Font: TGuiFont; Text: string);
  var
    OutPos: TVec2f;
begin
  if Font = nil then
    Exit;
  OutPos := Pos - Align * Vec2f(Font.GetWidth(Text), Font.GetHeight(Text));
  Gui.Graph.SetBlendMode(bmAlpha);
  Font.Draw(OutPos, Text);
end;

procedure TGuiStyle.AddFont(Name, FileName: String; Spacing: Integer = 0);
  var
    I: Integer;
begin
  for I := 0 to FFonts.Last do
    if FFonts[I].FName = Name then begin
      //FFonts[I].Free;
      // ...;
      Exit;
    end;
  FFonts.Add(TGuiFont.Create(Gui.FontMan, FileName, Spacing));
  FFonts[FFonts.Last].FStyle := Self;
  FFonts[FFonts.Last].FName := Name;
end;

function TGuiStyle.FindFont(Name: String): TGuiFont;
  var
    I: Integer;
begin
  Result := nil;
  for I := 0 to FFonts.Last do
    if FFonts[I].Name = Name then begin
      Result := FFonts[I];
      Exit;
    end;
end;
{$ENDREGION}

{$REGION 'TGui'}
function TGui.GetMouse: TVec2f;
begin
  Result := FWindow.Mouse - FRoot.Global;
end;

function TGui.RecOnMouseDown;
  var
    I: Integer;
begin
  Result := True;
  for I := Widget.Sub.Last downto 0 do
    if RecOnMouseDown(Widget.Sub[I], Button, Pos, Flags) then
      Exit;
  if not Widget.OnMouseDown(Button, Pos, Flags) then
    Result := False;
end;

constructor TGui.Create;
  var
    Style: TGuiStyle;
begin
  FGraph := Graph;
  FWindow := Window;
  FFontMan := TFontMan.Create(FGraph);
  FStyles := TListOfGuiStyle.Create;
  Style := TGuiStyle.Create(Self, '__default__');
  with Style do begin
    PrimaryColor := Vec4f(1, 1, 1, 1);
    SecondaryColor := Vec4f(1.0, 0.5, 0.5, 1.0);
    BackColor := Vec4f(0, 0, 0, 1);
    DisabledColor := Vec4f(0.5, 0.5, 0.5, 1.0);
  end;
  AddStyle(Style);
  FRoot := TWidget.Create(Self, '__root__', '__default__');
  FRoot.LocalRect := Rect(Vec2f(0, 0), FGraph.ScreenSize);
end;

destructor TGui.Destroy;
  var
    I: Integer;
begin
  for I := 0 to FStyles.Last do
    FStyles[I].Free;
  FStyles.Free;
  FFontMan.Free;
  FRoot.Free;
end;

procedure TGui.Render;
begin
  FRoot.OnRender;
end;

procedure TGui.Update;
begin
  FRoot.OnUpdate;
end;

procedure TGui.OnEvent(Event: TEvent);
begin
  if Event is TWinApiMouse then
    with TWinApiMouse(Event) do
      if Down then
        OnMouseDown(Button, Pos, Flags);
end;

procedure TGui.OnMouseDown(Button: TMouseButton; Pos: TVec2f; Flags: Cardinal);
begin
  RecOnMouseDown(FRoot, Button, Pos, Flags);
end;

procedure TGui.AddStyle(Style: TGuiStyle);
begin
  FStyles.Add(Style);
end;

function TGui.FindStyle(Name: String): TGuiStyle;
  var
    I: Integer;
begin
  Result := FStyles[0];
  for I := 0 to FStyles.Last do
    if FStyles[I].Name = Name then begin
      Result := FStyles[I];
      Exit;
    end;
end;

procedure TGui.Link(Parent, Widget: TWidget);
begin
  Unlink(Widget);
  if Parent = nil then
    Exit;
  Parent.Sub.Add(Widget);
  Widget.FParent := Parent;
end;

procedure TGui.Unlink(Widget: TWidget);
begin
  if Widget.Parent <> nil then
    Widget.Parent.Sub.Del(Widget, True);
  Widget.FParent := nil;
end;

procedure TGui.Popup(Widget: TWidget);
  var
    Parent: TWidget;
    I: Integer;
begin
  Parent := Widget.Parent;
  if Widget.Parent <> nil then begin
    Parent.Sub.Del(Widget, True);
    Parent.Sub.Add(Widget);
  end;
end;

function TGui.Trace(Point: TVec2f): TWidget;
begin
  Result := FRoot.Trace(Point);
end;

procedure TGui.SetFocus(Widget: TWidget);
begin
  FFocus := Widget;
end;
{$ENDREGION}

{$REGION 'TWidget'}
function TWidget.GetGlobal: TVec2f;
begin
  if FParent = nil then
    Result := FLocal
  else
    Result := FParent.Global + FLocal;
end;

procedure TWidget.SetGlobal(NewPos: TVec2f);
begin
  if FParent = nil then
    FLocal := NewPos
  else
    FLocal := NewPos - FParent.Global;
end;

function TWidget.GetLocalCenter: TVec2f;
begin
  Result := (LocalRect.A + LocalRect.B)/2;
end;

procedure TWidget.SetLocalCenter(NewCenter: TVec2f);
begin
  Local := NewCenter - Size/2;
end;

function TWidget.GetGlobalCenter: TVec2f;
begin
  Result := (GlobalRect.A + GlobalRect.B)/2;
end;

procedure TWidget.SetGlobalCenter(NewCenter: TVec2f);
begin
  Global := NewCenter - Size/2;
end;

function TWidget.GetLocalRect: TRect;
begin
  Result := DMath.Rect(FLocal, FLocal + FSize);
end;

procedure TWidget.SetLocalRect(NewRect: TRect);
begin
  Local := NewRect.A;
  Size := NewRect.B - NewRect.A;
end;

function TWidget.GetGlobalRect: TRect;
begin
  if FParent = nil then
    Result := LocalRect
  else begin
    Result.A := Global;
    Result.B := Result.A + Size;
  end;
end;

procedure TWidget.SetGlobalRect(NewRect: TRect);
begin
  Global := NewRect.A;
  Size := NewRect.B - NewRect.A;
end;

constructor TWidget.Create;
begin
  FGui := Gui;
  FName := Name;
  FStyle := FGui.Styles[Style];
  FFocusable := False;
  FParent := nil;
  FSub := TListOfWidget.Create;
  FShow := True;
end;

destructor TWidget.Destroy;
begin
end;

procedure TWidget.SetParent(Parent: TWidget);
begin
  FGui.Link(Parent, Self);
end;

function TWidget.IsLocalPointIn(Point: TVec2f): Boolean;
begin
  Result := Intersect(LocalRect, Point);
end;

function TWidget.IsGlobalPointIn(Point: TVec2f): Boolean;
begin
  Result := Intersect(GlobalRect, Point);
end;

function TWidget.Trace(Point: TVec2f): TWidget;
  var
    I: Integer;
begin
  for I := FSub.Last downto 0 do begin
    Result := FSub[I].Trace(Point);
    if Result <> nil then
      Exit;
  end;
  if IsGlobalPointIn(Point) then
    Result := Self
  else
    Result := nil;
end;

function TWidget.IsMouseIn: Boolean;
begin
  Result := True;
  if FParent <> nil then
    Result := FParent.IsMouseIn;
  Result := Result and IsGlobalPointIn(Gui.Mouse);
end;

function TWidget.OnMouseDown(Button: TMouseButton; Pos: TVec2f; 
                             Flags: Cardinal): Boolean;
begin
  Result := False;
end;

procedure TWidget.OnRender;
  var
    I: Integer;
begin
  Render;
  for I := 0 to FSub.Last do
    if FSub[I].Show then 
      FSub[I].OnRender;
end;

procedure TWidget.OnUpdate;
  var
    I: Integer;
begin
  Update;
  for I := 0 to FSub.Last do
    if FSub[I].Show then
      FSub[I].Update;
end;

procedure TWidget.SetFocus;
begin
  if FFocusable then
    Gui.SetFocus(Self)
  else
    Gui.SetFocus(nil);
end;

procedure TWidget.Render;
begin
end;

procedure TWidget.Update;
begin
end;
{$ENDREGION}

function TWidgetDecorator.GetGui: TGui;
begin
  Result := Widget.Gui;
end;

function TWidgetDecorator.GetStyle: TGuiStyle;
begin
  Result := Widget.Style;
end;

constructor TWidgetDecorator.Create(Widget: TWidget);
begin
  FWidget := Widget;
end;

end.
