unit DMeta;

interface

uses
  //objpas,
  DUtils,
  DDebug;

const
  STRING_FLAG           = 1;
  INTEGER_FLAG          = 2;
  FLOAT_FLAG            = 4;
  FUNC_FLAG             = 8;

type
TMetaFunc = function(Param: Pointer): Pointer of object;

TMetaVarType =  (
                        mvtInt,
                        mvtStr,
                        mvtFlt,
                        mvtFunc
                );

type
TMetaVar = class(TObject)
private
  FType: TMetaVarType;
  FInteger: Integer;
  FString: String;
  FFloat: Single;
  FFunc: TMetaFunc;
  FChanges: Integer;
  FOnChange: TProcObj;
  procedure DefOnChange;
public
  constructor Create;
  function DefMetaFunc(Param: Pointer): Pointer;
  procedure SetInteger(Value: Integer); inline;
  procedure SetString(Value: String); inline;
  procedure SetFloat(Value: Float); inline;
  procedure SetFunc(Value: TMetaFunc); inline;
  function GetInteger: Integer; inline;
  function GetString: String; inline;
  function GetFloat: Single; inline; 
  function GetFunc: TMetaFunc; inline;
end;

TMetaNamedVar = record
  Name: string;
  MetaVar: TMetaVar;
end;

TMetaTable = class
private
  FVars: array of TMetaNamedVar;
  function GetVar(Name: string; var MetaVar: TMetaVar): Boolean; overload;
public
  function GetVar(Name: string): TMetaVar; overload; inline;
  function GetVar(Name: string; Def: Integer): TMetaVar; overload; inline;
  function GetVar(Name: string; Def: string): TMetaVar; overload; inline;
  function GetVar(Name: string; Def: Float): TMetaVar; overload; inline;
  procedure DelVar(Name: string);
  function VarCount: Integer; inline;
  procedure PrintAllValues;
  function GetName(ID: Integer): string;
  function GetValue(ID: Integer): TMetaVar;
  property Names[ID: Integer]: string read GetName;
  property Values[ID: Integer]: TMetaVar read GetValue;
  property Vars[Name: string]: TMetaVar read GetVar; default;
end;

implementation

//{{{ TMetaVar
procedure TMetaVar.DefOnChange;
begin
end;

constructor TMetaVar.Create;
begin
  FOnChange := DefOnChange;
end;

function TMetaVar.DefMetaFunc(Param: Pointer): Pointer;
begin
  Result := nil;
end;

procedure TMetaVar.SetInteger(Value: Integer);
begin
  if (FType <> mvtInt) or (FInteger <> Value) then
    FOnChange;
  FType := mvtInt;
  FInteger := Value;
  FChanges := not INTEGER_FLAG;
end;

procedure TMetaVar.SetString(Value: String);
begin
  if (FType <> mvtStr) or (FString <> Value) then
    FOnChange;
  FType := mvtStr;
  FString := Value;
  FChanges := not STRING_FLAG;
end;

procedure TMetaVar.SetFloat(Value: Float); 
begin
  if (FType <> mvtFlt) or (FFloat <> Value) then
    FOnChange;
  FType := mvtFlt;
  FFloat := Value;
  FChanges := not FLOAT_FLAG;
end;

procedure TMetaVar.SetFunc(Value: TMetaFunc);
begin
  if (FType <> mvtFunc) or (@FFunc <> @Value) then
    FOnChange;
  FType := mvtFunc;
  FFunc := Value;
  FChanges := not FUNC_FLAG;
end;

function TMetaVar.GetInteger: Integer;
begin
  if FChanges and INTEGER_FLAG > 0 then begin
    case FType of
      mvtStr: FInteger := StrToIntDef(FString, 0);
      mvtFlt: FInteger := Trunc(FFloat);
    end;
    FChanges := FChanges and not INTEGER_FLAG;
  end;
  Result := FInteger;
end;

function TMetaVar.GetString: String; 
begin
  if FChanges and STRING_FLAG > 0 then begin
    case FType of
      mvtInt: FString := IntToStr(FInteger);
      mvtFlt: FString := FloatToStr(FFloat);
    end;
    FChanges := FChanges and not STRING_FLAG; 
  end;
  Result := FString;
end;

function TMetaVar.GetFloat: Single; 
begin
  if FChanges and FLOAT_FLAG > 0 then begin
    case FType of
      mvtInt: FFLoat := FInteger;
      mvtStr: FFloat := StrToFloat(FString);
      mvtFunc: FFloat := 0;
    end;
    FChanges := FChanges and not FLOAT_FLAG;
  end;
  Result := FFloat;
end;

function TMetaVar.GetFunc: TMetaFunc;
begin
  if FChanges and FUNC_FLAG > 0 then begin
    FFunc := DefMetaFunc;
    FChanges := FChanges and not FUNC_FLAG;
  end;
  Result := FFunc;
end;
//}}}

//{{{ TMetaTable
function TMetaTable.GetVar(Name: string; var MetaVar: TMetaVar): Boolean;
  var
    I: Integer;
begin
  for I := 0 to High(FVars) do
    if Name = FVars[I].Name then begin
      MetaVar := FVars[I].MetaVar;
      Result := False;
      Exit;
    end;
  SetLength(FVars, Length(FVars) + 1);
  FVars[High(FVars)].Name := Name;
  FVars[High(FVars)].MetaVar := TMetaVar.Create;
  MetaVar := FVars[High(FVars)].MetaVar;
end;

function TMetaTable.GetVar(Name: string): TMetaVar;
begin
  GetVar(Name, Result);
end;

function TMetaTable.GetVar(Name: string; Def: Integer): TMetaVar;
begin
  if GetVar(Name, Result) then
    Result.SetInteger(Def);
end;

function TMetaTable.GetVar(Name: string; Def: string): TMetaVar;
begin
  if GetVar(Name, Result) then
    Result.SetString(Def);
end;

function TMetaTable.GetVar(Name: string; Def: Float): TMetaVar;
begin
  if GetVar(Name, Result) then
    Result.SetFloat(Def);
end;

function TMetaTable.VarCount;
begin
  Result := Length(FVars);
end;

function TMetaTable.GetName(ID: Integer): string;
begin
  Result := FVars[ID].Name;
end;

function TMetaTable.GetValue(ID: Integer): TMetaVar;
begin
  Result := FVars[ID].MetaVar;
end;

procedure TMetaTable.DelVar(Name: string);
  var
    I: Integer;
begin
  for I := 0 to High(FVars) do
    if FVars[I].Name = Name then begin
      FVars[I].MetaVar.Free;
      FVars[I] := FVars[High(FVars)];
      Exit;
    end;
end;

procedure TMetaTable.PrintAllValues;
  var
    I: Integer;
begin
  for I := 0 to High(FVars) do
    Log.PrintLn(FVars[I].Name + ': ' + FVars[I].MetaVar.GetString);
end;
//}}}

end.
