unit ResCore;

interface

Uses
  DDebug,
  DUtils,
  DCore;

Type
TResource = class(TEntity)
{private
  FName: String;
public
  Constructor Create(Owner: TEntity); override;
  Function IsEqual(Other: TResource): Boolean; virtual;
  Property Name: String read FName write FName;}
end;

TLoader = class(TEntity)
  Function Load(FileName: String; Data: TData): TResource; virtual; 
end;

TManager = class(TEntity)
private
  FLoaders: TList;
  FResources: TNamedList;
public
  Constructor Create(Owner: TEntity); override;
  Destructor Destroy; override;
  Procedure RegistLoader(Loader: TLoader);
  Function Load(FileName: String): TResource;
end;

implementation

{//
//  TResource
//
Constructor TResource.Create(Owner: TEntity);
begin
  Inherited Create(Owner);
  FName := '';
end;

Function TResource.IsEqual(Other: TResource): Boolean;
begin
  Result := (ClassName = Other.ClassName)And
            (FName = Other.FName);
end;}

//
//  TLoader
//
Function TLoader.Load(FileName: String; Data: TData): TResource;
begin
  Result := Nil;
end;

//
//  TManager
//
Constructor TManager.Create;
begin
  Inherited Create(Owner);
  FLoaders := TList.Create(Self);
  FResources := TNamedList.Create(Self);
end;

Destructor TManager.Destroy;
begin
  FLoaders.FreeAll;
  FLoaders.Free;
  FResources.FreeAll;
  FResources.Free;
  Inherited Destroy;
end;

Procedure TManager.RegistLoader;
begin
  FLoaders.Add(Loader);
end;

Function TManager.Load(FileName: String): TResource;
  Var
    Data: TData;
    I: Integer;
begin
    If FResources[FileName] <> Nil then begin
      Result := TResource(FResources[FileName]);
      Exit;
    end;
  Result := Nil;
    If Not FileExist(FileName) then
      Data := Nil
    else
      Data := TData.Create(FileName);
  I := 0;
    While (Result = Nil)And(I < FLoaders.Length) do begin
      Result := TLoader(FLoaders[I]).Load(FileName, Data);
      Inc(I);
    end;
    If Result <> Nil then
      FResources[FileName] := Result;
    If Data <> Nil then
      Data.Free;
end;

end.
