unit _Objects;

interface

Uses
  Windows,
  Use_OpenGl,
  opengl,
  Global,
  Use_Geometry,
  Use_3d;

const
  Max_Units = 64;
  Max_Trees = 250;

type

  TUnit = record
    tip:byte; {0-none}
    Side:byte;{0-neytral 1-white 2-black}
    Life:integer;
    {moved}
    ang,need_ang:single;
    moved:boolean;{unit can move one time for turn}
    fired:boolean;{unit can fire one time for turn}
    Speed:integer;{how any cells can move unit for one turn}
    {fire}
    Damage:byte;
    Radius:single;
    {anim}
    anim,fool_anim:integer;
    cx,cy:single;
  end;

  TMapCell = record
    Army:integer;
    Mask:boolean;
    Attack_Mask:boolean;
    Cost:integer;
    Attack_Cost,  {for ai}
    Moving_Cost,
    Defence_cost:integer;
  end;

  TTree = record
    Point:TVect;
    Tip:byte;
    faze:byte;
  end;

  TChessBoard = class
    private
    Tree:array [0..Max_Trees] of TTree;
    procedure Put_Unit(X,Y,Z:single;tip,side,Life,anim:byte;Mash,ang:single);
    procedure Clear_Mask;
    procedure Clear_Units_From_Mask;       
    procedure Clear_Attack_Mask;
    procedure Show(x,y:byte;len:integer;first:boolean);
    procedure Show_Ways(x,y,unit_tip:byte);
    procedure Show_Attacked(x,y,unit_Rad:Single);
    procedure creation_map_anim();
    procedure Put_Trees(X,Y:single;tip:byte;fase:byte);
    public
    Sel,SX,SY:integer;{selected unit}
    {props}
    Show_Shadows,
    Show_Trees,
    Show_Lifes:boolean;
    AI_Side,
    Current_Side,
    All_Sides:byte;{all-2 player -0}
    Map_Paint_anim,
    Max_Map_PA:integer;{map creation animate}
    Map:array[0..9,0..9] of TMapCell;
    Units:array [0..Max_Units] of TUnit;
    constructor Create();
    procedure Make_Turn;
    procedure Render();
    procedure Anim();
    procedure Move(DX,DY:integer;mbl,mbr:boolean);
    procedure AI_Engine();
    function Add_Unit(X,Y:byte;tip,side:byte):integer;    
    procedure Save_Map(FileName:String);
    procedure Load_Map(FileName:String);
  end;
                         
Var
  Map:TChessBoard;

implementation

procedure TChessBoard.Save_Map(FileName:String);
Var
 F:file of char;
 s,t:char;
 i,j:byte;
begin
 AssignFile(F,FileName);
 Rewrite(F);
 for i := 1 to 8 do
  begin
   for j := 1 to 8 do
   begin
    t:='0';
    s:=' ';
    if Self.Map[i,j].Army>0 then
    begin
     t:=chr(Self.Units[Self.Map[i,j].Army].tip+64);
     s:=chr(Self.Units[Self.Map[i,j].Army].Side+48);
    end;
    Write(F,t);
    Write(F,s);
    s:=' ';      
    Write(F,s);
   end;
   t:=chr(13);
   Write(F,t);
  end;
 CloseFile(F);
end;

procedure TChessBoard.Load_Map(FileName:String);
Var
 F:file of char;
 t:char;
 i,j,tip:byte;
begin
 AssignFile(F,FileName);
 for i:=1 to 8 do
  for j:=1 to 8 do
   Map[i,j].Army:=0;
 for j:=1 to Max_Units do
  Units[i].tip:=0;
 Reset(F);
 i:=1;
 j:=1;
 Repeat
  case t of
   '0',' ':;//none
   '1':Self.Add_Unit(i,j,tip,1);   //sides
   '2':Self.Add_Unit(i,j,tip,2);
   'A'..'Z':tip:=ord(t)-64;
  end;
 i:=i+1;
 if i>8 then begin i:=1;j:=j+1; end;
 until not eof(F);
 CloseFile(F);
end;

procedure TChessBoard.Put_Trees(X,Y:single;tip:byte;fase:byte);
Var
  i:byte;
  M,SH:TMatrix4f;
begin                   
 glEnable(GL_STENCIL_TEST);
 glStencilFunc(GL_EQUAL,0,1);
 glStencilOp(GL_KEEP,GL_KEEP,GL_DECR);
 M:=Pozition(Default_TMatrix,X,Y,0);
 if fase=255 then exit;
 glPushMatrix();
 glMultMatrixf(@M);
 case tip of
  1:begin
      Tree1.Anim(2-fase/50);
      Tree1.Render(Tree1.Texture);
    end;
  2:begin
      Tree2.Anim(2-fase/50);
      Tree2.Render(Tree1.Texture);
    end;
  3:begin
      Tree3.Anim(2-fase/50);
      Tree3.Render(Tree1.Texture);
    end;
 end;
 glPopMatrix(); 
 if Show_Shadows then
 begin
  glPushMatrix();
  glColorMask(false,false,false,false);
  glStencilFunc(GL_EQUAL,0,1);
  glStencilOp(GL_KEEP,GL_KEEP,GL_INCR);
  SH:=M;
  SH[0,2]:=0;
  SH[1,2]:=0;
  SH[2,2]:=0;
  SH[3,2]:=0;
  SH[2,0]:=SH[2,0]+1;
  SH[2,1]:=SH[2,1]+0.2;
  glMultMatrixf(@SH);  {!Shadow}
  case tip of
   1:Tree1.Flat_Shadow_Render;
   2:Tree2.Flat_Shadow_Render;
   3:Tree3.Flat_Shadow_Render;
  end;                
  glColorMask(true,true,true,true);
  glStencilFunc(GL_EQUAL,1,1);
  glStencilOp(GL_KEEP,GL_KEEP,GL_KEEP);
  glDisable(GL_STENCIL_TEST);
  glPopMatrix();
 end;
end;

procedure TChessBoard.Put_Unit(X,Y,Z:single;tip,side,life,anim:byte;Mash,ang:single);
Var
  a,b:byte;
  S:single;
  M,SH:TMatrix4f;
begin
 glEnable(GL_STENCIL_TEST);
 glStencilFunc(GL_EQUAL,0,1);
 glStencilOp(GL_KEEP,GL_KEEP,GL_DECR);
 a:=(tip-1) mod 4; {}
 b:=trunc((tip-1)/4);
 if Mash=1 then
   M:=Pozition(Rotate(ang,0,0,1),X+0.5,Y+0.5,Z) else {default position}
  M:=Pozition(multi(Matrix(Mash,0,0,0,Mash,0,0,0,Mash),Rotate(ang,0,0,1)),
      X+0.5,Y+0.5,Z); {scaled pozition (for creation)}
 glPushMatrix();
 glMultMatrixf(@M);
 case tip of
  0:;
  1:begin
      if anim=0 then Sold.Anim(0) else
       Sold.anim(7-(anim mod (6*4))/4);
      Sold.Render(Sold.Texture);
    end;
  2:tank.Render(Tank.Texture); {tank}
  3:car.Render(car.Texture); {car}
  4:fort.Render(Fort.Texture); {fort}
  5:art.Render(art.Texture); {artillery}
 end;
 glPopMatrix();
 if Show_Shadows then
 begin
  glPushMatrix();
  glColorMask(false,false,false,false);
  glStencilFunc(GL_EQUAL,0,1);
  glStencilOp(GL_KEEP,GL_KEEP,GL_INCR);
  SH:=M;
  SH[0,2]:=0;
  SH[1,2]:=0;
  SH[2,2]:=0;
  SH[3,2]:=0;
  SH[2,0]:=SH[2,0]+1;
  SH[2,1]:=SH[2,1]+0.2;
  glMultMatrixf(@SH);  {!Shadow}
  case tip of
   0:;
   1:Sold.Flat_Shadow_Render;
   2:tank.Flat_Shadow_Render;  {tank}
   3:car.Flat_Shadow_Render;  {car}
   4:fort.Flat_Shadow_Render;     {fort}
   5:art.Flat_Shadow_Render;    {artillery}
  end;
  glColorMask(true,true,true,true);
  glStencilFunc(GL_EQUAL,1,1);
  glStencilOp(GL_KEEP,GL_KEEP,GL_KEEP);
  glDisable(GL_STENCIL_TEST);
  glPopMatrix();
 end;
 {lifes view}
 if life>0 then
 begin
  glPushMatrix();    
  glDisable(gl_Texture_2d);
  {billboard}
  M:=Pozition(Matrix(1,0,0,0,1,0,0,0,1),X+0.5,Y+0.5,Z-0.8);
  glMultMatrixf(@M);
  glMultMatrixf(@Back_view);
  case side of
   0:glColor3f(1,1,1);{neytral}
   1:glColor3f(0,5,0);{alied}
   2:glColor3f(0,0,5);{axies}
  end;
  glbegin(gl_Quads);
   for a:=0 to Life do
   begin
    S:=a*0.07-(Life*0.07)/2;
    glVertex3f(S-0.03,-0.03,0);
    glVertex3f(S+0.03,-0.03,0);
    glVertex3f(S+0.03, 0.03,0);
    glVertex3f(S-0.03, 0.03,0);
   end;
  glend();
  glPopMatrix();
 end;
end;

procedure TChessBoard.creation_map_anim();
Var i,j:byte;Z:single;
begin
 {$Region animation on map create}
 glDisable(GL_Depth_Test);
 glbegin(gl_Quads);
  glColor3f(0.9,0.9,0.6);
//  glColor3f(0.9,0.9,0.9);
  glVertex3f(-200,-200,0.5); glVertex3f( 200,-200,0.5);
  glVertex3f( 200, 200,0.5); glVertex3f(-200, 200,0.5);
 glend();
  //lColor3f(0.5,0.5,0.2);
{zakras line}
 Z:=10-10*(abs(Map_Paint_anim)-40)/Max_Map_PA;
 glColor3f(0.5,0.5,0.1);
 glbegin(gl_Quads);
  glVertex3f( 0  ,-10 ,0.3);
  glVertex3f(-10  , 0 ,0.3);
  glVertex3f(-10+Z, 0+Z,0.3);
  glVertex3f( 0+Z,-10+Z,0.3);
 glend();
 {kletki_small}
 glColor3f(0.9,0.9,0.6);
// glColor3f(0.9,0.9,0.9);
 glbegin(gl_Quads);
  glVertex3f((0.73-KX),(0.73-KY),0);
  glVertex3f((9.27-KX),(0.73-KY),0);
  glVertex3f((9.27-KX),(1-KY),0);
  glVertex3f((0.73-KX),(1-KY),0);
  glVertex3f((0.73-KX),(9  -KY),0);
  glVertex3f((9.27-KX),(9  -KY),0);
  glVertex3f((9.27-KX),(9.27-KY),0);
  glVertex3f((0.73-KX),(9.27-KY),0);
  glVertex3f((0.73-KX),(0.73-KY),0);
  glVertex3f((1  -KX),(0.73-KY),0);
  glVertex3f((1  -KX),(9.27-KY),0);
  glVertex3f((0.73-KX),(9.27-KY),0);
  glVertex3f((9  -KX),(0.73-KY),0);
  glVertex3f((9.27-KX),(0.73-KY),0);
  glVertex3f((9.27-KX),(9.27-KY),0);
  glVertex3f((9  -KX),(9.27-KY),0);
 glend();
 glbegin(gl_Quads);
 for i := 1 to 8 do
  for j := 1 to 8 do
   begin
    if (i+j) mod 2<>0 then
    begin
      glColor3f(0.9,0.9,0.6);
 //    glColor3f(0.9,0.9,0.9);     {white cells}
     glVertex3f((i-KX+0.03),(j-KY+0.03),0);    glVertex3f((i-KX+0.97),(j-KY+0.03),0);
     glVertex3f((i-KX+0.97),(j-KY+0.97),0);    glVertex3f((i-KX+0.03),(j-KY+0.97),0);
    end;
   end;
 glend();
{zakras plane}
 Z:=10-10*(abs(Map_Paint_anim))/Max_Map_PA;
 glColor3f(0.5,0.5,0.2);
// glColor3f(0.2,0.2,0.2);
 glbegin(gl_Quads);
  glVertex3f( 0  ,-10 ,0);
  glVertex3f(-10  , 0 ,0);
  glVertex3f(-10+Z, 0+Z,0);
  glVertex3f( 0+Z,-10+Z,0);
 glend();
 {kletki}
 glColor3f(0.9,0.9,0.6);
// glColor3f(0.9,0.9,0.9);
 glbegin(gl_Quads);
  glVertex3f((0.7-KX),(-20),0);    glVertex3f((9.3-KX),(-20),0);
  glVertex3f((9.3-KX),(0.7-KY),0); glVertex3f((0.7-KX),(0.7-KY),0);
  glVertex3f((0.7-KX),(9.3-KY),0); glVertex3f((9.3-KX),(9.3-KY),0);
  glVertex3f((9.3-KX),(20),0);     glVertex3f((0.7-KX),(20),0);
  glVertex3f(( -20),(-20),0);      glVertex3f((0.7-KX),(-20),0);
  glVertex3f((0.7-KX),(20),0);     glVertex3f((   -20),(20),0);
  glVertex3f((9.3-KX),(-20),0);    glVertex3f((    20),(-20),0);
  glVertex3f((  20),(20),0);       glVertex3f((9.3-KX),(20),0);
 glend();
 glbegin(gl_Quads);
 for i := 1 to 8 do
  for j := 1 to 8 do
   begin
    if ((i+j) mod 2=0) then
     begin
      glVertex3f((i-KX  ),(j-KY  ),0); glVertex3f((i-KX+1),(j-KY  ),0);
      glVertex3f((i-KX+1),(j-KY+1),0); glVertex3f((i-KX  ),(j-KY+1),0);
     end;
   end;
 glend();
 glColor4f(5,5,5,5);
 glEnable(GL_Depth_Test);
// glEnable(GL_BLEND);
 glEnable(GL_Texture_2d);
 glBindTexture(GL_Texture_2D,_Units);
 for i := 1 to 8 do
  for j := 1 to 8 do
   begin
    if (Map[i,j].Army>0) then
     with Units[Map[i,j].Army] do
      begin
        Z:=(-(i+j)*10-abs(Map_Paint_anim)-20+Max_Map_PA)/12;
        if Z>1 then Z:=1;
        if Z>0 then Put_Unit(i+CX*Anim-KX,j+CY*Anim-KY,0,
          Units[Map[i,j].Army].tip,Units[Map[i,j].Army].Side,
                0,0,z,Units[Map[i,j].Army].ang);
      end;
   end;
 for i := 0 to Max_Trees do {trees}
 begin
  Z:=(-(Tree[i].Point[0]+Tree[i].Point[1])-abs(Map_Paint_anim)-20+Max_Map_PA);
  if ((Z)*5>=0) and (Tree[i].faze=255) then Tree[i].faze:=120;
  Put_Trees(Tree[i].Point[0]-KX,Tree[i].Point[1]-KY,Tree[i].Tip,Tree[i].faze);
 end;
 {$endregion}
end;

procedure TChessBoard.Render();
var
  i,j,z:byte;
  M:TMatrix4f;
begin
 {rost palm}
 for i:=0 to Max_Trees do
  if (Tree[i].faze>0) and (Tree[i].faze<255) then
   begin
    Tree[i].faze:=Tree[i].faze-1;
   end;
 {/rost palm}
 glDisable(GL_Texture_2d);
 glDisable(GL_Depth_Test);
 if (Map_Paint_anim=0) then {esli karta ne prorisovalvas - otobragat prorisovku}
 begin
 {podlogka i kromki}
 glbegin(gl_Quads);
 // glColor3f(1,1,0.5);
  glColor3f(0.9,0.9,0.6);
  glVertex3f(-200,-200,0.1); glVertex3f( 200,-200,0.1);
  glVertex3f( 200, 200,0.1); glVertex3f(-200, 200,0.1);
  glColor3f(0.5,0.5,0.2);
  //lColor3f(0.5,0.5,0.2);
  glVertex3f((0.7-KX),(0.7-KY),0);   glVertex3f((9.3-KX),(0.7-KY),0);
  glVertex3f((9.3-KX),(1  -KY),0);   glVertex3f((0.7-KX),(1  -KY),0);
  glVertex3f((0.7-KX),(9  -KY),0);   glVertex3f((9.3-KX),(9  -KY),0);
  glVertex3f((9.3-KX),(9.3-KY),0);   glVertex3f((0.7-KX),(9.3-KY),0);
  glVertex3f((0.7-KX),(0.7-KY),0);   glVertex3f((1  -KX),(0.7-KY),0);
  glVertex3f((1  -KX),(9.3-KY),0);   glVertex3f((0.7-KX),(9.3-KY),0);
  glVertex3f((9  -KX),(0.7-KY),0);   glVertex3f((9.3-KX),(0.7-KY),0);
  glVertex3f((9.3-KX),(9.3-KY),0);   glVertex3f((9  -KX),(9.3-KY),0);
 glend();
 {board}
 glbegin(gl_Quads);
 for i := 1 to 8 do
  for j := 1 to 8 do
   begin
    if (i+j) mod 2=0 then
     //glColor3f(1,1,0.5)
     glColor3f(0.9,0.9,0.6) else {white and}
     glColor3f(0.5,0.5,0.2);
     //glColor3f(0.5,0.5,0.2);     {dark cells}
    {cell}
    glVertex3f((i-KX  ),(j-KY  ),0);
    glVertex3f((i-KX+1),(j-KY  ),0);
    glVertex3f((i-KX+1),(j-KY+1),0);
    glVertex3f((i-KX  ),(j-KY+1),0);
    if Map[i,j].Mask then
     begin  {zone to move}
      glColor4f(0.0,2.5,0.0,0.5);
      glVertex3f((i-KX  ),(j-KY  ),0); glVertex3f((i-KX+1),(j-KY  ),0);
      glVertex3f((i-KX+1),(j-KY+1),0); glVertex3f((i-KX  ),(j-KY+1),0);
     end;
    if Map[i,j].Attack_Mask then
     begin  {zone to attack}
      glColor4f(1.5,0.5,0.5,0.8);
      glVertex3f((i-KX  ),(j-KY  ),0); glVertex3f((i-KX+1),(j-KY  ),0);
      glVertex3f((i-KX+1),(j-KY+1),0); glVertex3f((i-KX  ),(j-KY+1),0);
     end;
    if (DX>=i)and (DX-1<i) and (DY>=j)and (DY-1<j) then
     begin  {selection}
      glColor3f(2,0.5,0.5);
      glVertex3f((i-KX  ),(j-KY  ),0); glVertex3f((i-KX+1),(j-KY  ),0);
      glVertex3f((i-KX+1),(j-KY+1),0); glVertex3f((i-KX  ),(j-KY+1),0);
     end;
   end;
 glend();
 {units}
 glColor4f(5,5,5,5);
 glEnable(GL_Depth_Test);
// glEnable(GL_BLEND);
 {back matrix for billboard}
 M:=Pozition(Rotate(AX,0,0,-1));
 Back_view:=multi(Pozition(Rotate(AY+90,-1,0,0)),M);
 glEnable(GL_Texture_2d);
 glBindTexture(GL_Texture_2D,_Units);
 for i := 1 to 8 do
  for j := 1 to 8 do
   begin
    if (Map[i,j].Army>0) then
     with Units[Map[i,j].Army] do
      begin
        if Show_Lifes then z:=Units[Map[i,j].Army].Life else z:=0;
        if (Map[i,j].Army=Sel) or ((i=dx) and (j=dy)) then z:=Units[Map[i,j].Army].Life;
        Put_Unit(i+CX*Anim-KX,j+CY*Anim-KY,0,
          Units[Map[i,j].Army].tip,Units[Map[i,j].Army].Side,z,Units[Map[i,j].Army].anim,1,Units[Map[i,j].Army].ang);
      end;
   end;
 {palms}
 if self.Show_Trees then 
 for i := 0 to Max_Trees do
  Put_Trees(Tree[i].Point[0]-KX,Tree[i].Point[1]-KY,Tree[i].Tip,Tree[i].faze);
 {who turn?}
 case Current_Side of
 0:glColor3f(0.6,0.6,0.6);{neytral}
 1:glColor3f(0.1,1.5,0.1);{alied}
 2:glColor3f(0.1,0.1,1.5);{axies}
 end;
 glDisable(GL_Texture_2d);
 glPushMatrix();
 glLoadIdentity;
 glbegin(gl_Quads);
  glVertex3f(-5.3,3.3,0);
  glVertex3f(-5  ,3.3,0);
  glVertex3f(-5  ,3,0);
  glVertex3f(-5.3,3,0);
 glend();
 end else
 begin {map creation anim}
  Map_Paint_anim:=Map_Paint_anim-1;
  creation_map_anim;
 end;
 glLoadIdentity();
 {shadow paint}
 glColor4f(0,0,0,0.4);
 glEnable(GL_STENCIL_TEST);
 glStencilFunc(GL_EQUAL,1,1);
 glStencilOp(GL_KEEP,GL_KEEP,GL_KEEP);
 glDisable(GL_Depth_Test);
 glBegin(GL_QUADS);
  glVertex3f(-10,-10,-1);
  glVertex3f( 10,-10,-1);
  glVertex3f( 10,10, -1);
  glVertex3f(-10,10, -1);
 glend;        { }
 glEnable(GL_Depth_Test);
 glDisable(GL_STENCIL_TEST);

 glPopMatrix();
end;

procedure TChessBoard.Show(x,y:byte;len:integer;first:boolean);
begin
 if (x<=0) or (x>=9) or (y<=0) or (y>=9) then exit;
 if ((Map[x,y].Mask) and (Map[x,y].Cost>len))
  or (len<0) then exit;
 if ((Map[x,y].Army>0) and not first) then exit;
 Map[x,y].Mask:=true;
 Map[x,y].Cost:=len;
 Show(x+1,y,len-2,false);
 Show(x,y+1,len-2,false);
 Show(x-1,y,len-2,false);
 Show(x,y-1,len-2,false);
 Show(x+1,y+1,len-3,false);
 Show(x+1,y-1,len-3,false);
 Show(x-1,y-1,len-3,false);
 Show(x-1,y+1,len-3,false);  {  }
end;

procedure TChessBoard.Show_Ways(x,y,unit_tip:byte);
Var i,j:byte;
begin
 {show ways}
 Clear_Mask;
  case unit_tip of
   1:Show(x,y,3,true);{soldier}
   2:Show(x,y,5,true);{tank}
   3:Show(x,y,6,true);{spy}
   4:Show(x,y,-1,true);{bunker}{etc}
   5:Show(x,y,4,true);{artillery}{etc}
  end;
 Clear_Units_from_Mask;
end;

procedure TChessBoard.Show_Attacked(x,y,unit_Rad:Single);
Var i,j:byte;
begin
 {show ho can fire}
for i:=0 to 9 do
 for j:=0 to 9 do
  if (Map[i,j].Army>0) then
  if (Units[Map[i,j].Army].Side<>Current_Side) and (Get_S(x,y,i,j)<unit_Rad) then
    Map[i,j].Attack_Mask:=true else Map[i,j].Attack_Mask:=false
    else Map[i,j].Attack_Mask:=false;
end;

procedure TChessBoard.AI_Engine();
begin
 self.Make_Turn;
end;

procedure TChessBoard.Move(DX,DY:integer;mbl,mbr:boolean);
Var x,y,z:integer;
begin
 x:=DX;
 y:=DY;  { }
 if AI_Side=Current_Side then AI_Engine else
 begin
 if (x>0) and (x<9) and //in board
  (y>0) and (y<9) then
 begin
  {Selecting unit}
  if mbl then
   if (Map[x,y].Army>0)  then
   if (Units[Map[x,y].Army].Side=Current_Side) then
     begin
      Sel:=Map[x,y].Army;
      SX:=x;
      SY:=y;
      if not Units[Sel].moved then
       Show_Ways(SX,SY,Units[Sel].tip) else
       Clear_Mask;
      if (not Units[Sel].Fired) then
       Show_Attacked(SX,SY,Units[Sel].Radius);
     end else {unselect all}
     Sel:=-1 else
    Sel:=-1;
  {moving unit}
  if (mbr) and (Map[x,y].Army=0) and (Map[x,y].Mask) and (Sel>0) then
   //if {Is Can Go} then
    begin
     {adding of animation}
     Units[Sel].anim:=Round(Get_S(x,y,sx,sy)*20);
     Units[Sel].cx:=(sx-x)/Units[Sel].anim;
     Units[Sel].cy:=(sy-y)/Units[Sel].anim;
     Units[Sel].need_ang:=Get_R(90-Get_A(x,y,sx,sy));
     {moving of unit}
     Map[sx,sy].Army:=0;
     Map[x,y].Army:=Sel;
     sx:=x;
     sy:=y;
     Units[Sel].moved:=true;
     Clear_Mask;
     if (not Units[Sel].Fired) then
      Show_Attacked(SX,SY,Units[Sel].Radius);
    end;
  {fire unit}
  if (mbr) and (Map[x,y].Attack_Mask) and
     (Sel>0) and (not Units[Sel].Fired) then
   //if {Is Can Go} then
    begin
     {fire}
     z:=Map[x,y].Army;                               
     Units[Sel].need_ang:=Get_R(90-Get_A(x,y,sx,sy));
     Units[z].Life:=Units[z].Life-Units[Sel].Damage;
     Units[Sel].fired:=true;
     Units[Sel].moved:=true;
     {enemy unit destroy}
     if (Units[z].Life<=0) then
      begin
        Map[x,y].Army:=0;
        Units[z].tip:=0;
      end;
     Clear_Mask;
     Clear_Attack_Mask;
    end;
 end;
 end;
end;

procedure TChessBoard.Anim();
Var i:byte;
begin
 for i:=1 to Max_Units do
  if (Units[i].tip>0) then
  begin
   if (units[i].tip<>4) then {if not fortress}    
    Units[i].ang:=Get_R(Units[i].ang+
        Head_Rotate(Units[i].ang,Units[i].need_ang,10));
 //  if (abs(Get_R(Units[i].need_ang-Units[i].ang))>=6) then
    if Units[i].anim>0 then Units[i].anim:=Units[i].anim-1;
  end;
end;

procedure TChessBoard.Clear_Mask;
var i,j:byte;
begin
for i:=0 to 9 do
 for j:=0 to 9 do
  begin
   Map[i,j].Mask:=false;
   Map[i,j].Cost:=0;
  end;
end;

procedure TChessBoard.Clear_Attack_Mask;
var i,j:byte;
begin
for i:=0 to 9 do
 for j:=0 to 9 do
  begin
   Map[i,j].Attack_Mask:=false;
   Map[i,j].Cost:=0;
  end;
end;

procedure TChessBoard.Clear_Units_from_Mask;
var i,j:byte;
begin
for i:=0 to 9 do
 for j:=0 to 9 do
  if Map[i,j].Army>0 then
  begin
   Map[i,j].Mask:=false;
   Map[i,j].Cost:=0;
  end;
end;

function TChessBoard.Add_Unit(X,Y:byte;tip,side:byte):integer;
Var i:byte;
begin
 if (Map[x,y].Army=0) then
 for i:=1 to Max_Units do
  if (Units[i].tip=0) then
  begin
    Units[i].tip:=tip+1;
    Units[i].Side:=side;
    Units[i].CX:=0;
    Units[i].CY:=0;
    Units[i].anim:=0;
    Units[i].fool_anim:=0;
    case Units[i].tip of
     1:begin  {soldier}
        Units[i].Life:=2;
        Units[i].Damage:=1;
        Units[i].Speed:=3;
        Units[i].Radius:=1.8;
       end;
     2:begin  {tank}
        Units[i].Life:=4;
        Units[i].Damage:=3;
        Units[i].Speed:=5;
        Units[i].Radius:=1.8;
       end;
     3:begin  {car}
        Units[i].Life:=2;
        Units[i].Damage:=2;
        Units[i].Speed:=7;
        Units[i].Radius:=2.2;
       end;
     4:begin  {fortress}
        Units[i].Life:=5;
        Units[i].Damage:=3;
        Units[i].Speed:=-1;
        Units[i].Radius:=2.8;
       end;
     5:begin  {artillery}
        Units[i].Life:=2;
        Units[i].Damage:=4;
        Units[i].Speed:=4;
        Units[i].Radius:=3.4;
       end;
    end;
    Units[i].moved:=false;
    if side=1 then Units[i].need_ang:=180;
    Units[i].ang:=Units[i].need_ang;
    Map[x,y].Army:=i;
    break;
  end;
end;

constructor TChessBoard.Create();
Var i:byte;X,Y:Single;
begin
 Current_Side:=1;
// AI_Side:=2;
 AI_Side:=5;
 All_Sides:=2;
 Sel:=-1;
 Max_Map_PA:=200;
 for i:=0 to Max_Trees do
   begin
     Repeat
      if random(2)=0 then
       X:=(Random(40000)/1000)-15 else X:=(Random(20000)/1000)-6;
      if random(2)=0 then
       Y:=(Random(40000)/1000)-15 else Y:=(Random(20000)/1000)-6;
     until not ((X>0.7) and (X<9.3) and (Y>0.7) and (Y<9.3));
    Tree[i].Point:=Vector(X,Y);
    Tree[i].Tip:=Random(3)+1;
    Tree[i].faze:=255;
   end;
  Show_Shadows:=true;
  Show_Lifes:=true;
  Show_Trees:=true;
end;

procedure TChessBoard.Make_Turn;
Var i:byte;
begin
 Current_Side:=Current_Side+1;
 if Current_Side>All_Sides then Current_Side:=1;
 for i:=1 to Max_Units do
  begin
   Units[i].moved:=false;
   Units[i].fired:=false;
  end;
 Clear_Units_from_Mask;
 Clear_Mask;
end;

end.
