// XObject.cpp: implementation of the CXObject class.
//
//////////////////////////////////////////////////////////////////////

#include "XObject.h"

#include "D3DEngine.h"
//#include "Add\d3dapp.h"

extern LPDIRECT3DDEVICE9       g_pd3dDevice;
extern CD3DEngine Engine;

#define D3DVERTEX (D3DFVF_XYZ | D3DFVF_NORMAL | D3DFVF_TEX1 | D3DFVF_DIFFUSE)


// A structure for our custom vertex type
struct CUSTOMVERTEX
{
    FLOAT x, y, z, rhw; // The transformed position for the vertex
	DWORD color;        // The vertex color
};

// Our custom FVF, which describes our custom vertex structure
#define D3DFVF_CUSTOMVERTEX (D3DFVF_XYZRHW|D3DFVF_DIFFUSE)



//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CXObject::CXObject()
{
	Position = CVector(0,0,0);
	Angle = CVector(0,0,0);
	Scale = CVector(1,1,1);

//	AddAngle = CVector(0,0,0);

	//Visible = TRUE;
	Visible = FALSE;
	SecondTexture = NULL;
	bMultiTexture = FALSE;
	for (int i=0; i<7; i++){
		TextureScale[i] = CVector( 1.0f, 1.0f, 1.0f );
		TextureMode[i] = PL_TEX_NORMAL;
	}
	
	D3DXMatrixIdentity(&extMatrix);

}

CXObject::~CXObject()
{

}


void CXObject::SetPosition(CVector new_pos)
{
	Position = new_pos;
}

void CXObject::SetOrientation(CVector new_angle)
{
	Angle = new_angle;
}

void CXObject::SetScale(CVector new_scale)
{
	Scale = new_scale;
}


void CXObject::SetTextureMode(int N_tex, int new_mode, int t_add)
{
	TextureMode[N_tex] = new_mode;
	TextureAdd[N_tex] = t_add;
}

void CXObject::SetTextureScale(int N_tex, CVector new_scale)
{
	TextureScale[N_tex] = new_scale;
}


void CXObject::CalcBBox()
{
	//Mesh->D3DXComputeBoundingBox(NULL, Mesh->GetNumVertices(), NULL, &BBox_Min, &BBox_Max);
	BYTE* pVertices2;
  
	Mesh->LockVertexBuffer(D3DLOCK_READONLY, (LPVOID*)&pVertices2); 
    
    D3DXComputeBoundingBox( (D3DXVECTOR3*)pVertices2, Mesh->GetNumVertices(), 
        D3DXGetFVFVertexSize(Mesh->GetFVF()), &BBox_Min, &BBox_Max );
   
	Mesh->UnlockVertexBuffer();



	//LPDIRECT3DVERTEXBUFFER9 
	g_pVB        = NULL; // Buffer to hold vertices

	


	// Initialize three vertices for rendering a triangle
    CUSTOMVERTEX vertices[] =
    {
        { BBox_Min.x, BBox_Min.y, BBox_Min.z, 1.0f, 0xffff0000, }, // x, y, z, rhw, color
        { BBox_Max.x, BBox_Min.y, BBox_Min.z,  1.0f, 0xff00ff00, },
        { BBox_Max.x, BBox_Max.y, BBox_Min.z, 1.0f, 0xff00ffff, },
		{ BBox_Min.x, BBox_Min.y, BBox_Min.z, 1.0f, 0xff00ffff, },
		{ -20.0f, -20.0f, -20.0f, 1.0f, 0xff00ffff, },
		{ 20.0f, 20.0f, 20.0f, 1.0f, 0xff00ffff, },
    };

    // Create the vertex buffer. Here we are allocating enough memory
    // (from the default pool) to hold all our 3 custom vertices. We also
    // specify the FVF, so the vertex buffer knows what data it contains.
    if( FAILED( g_pd3dDevice->CreateVertexBuffer( 6*sizeof(CUSTOMVERTEX),
                                                  0, D3DFVF_CUSTOMVERTEX,
                                                  D3DPOOL_DEFAULT, &g_pVB, NULL ) ) )
    {
		MessageBox(NULL, "Cant Create BBox VBuffer", "CObject::CalcBBox()", MB_OK);
    }

    // Now we fill the vertex buffer. To do this, we need to Lock() the VB to
    // gain access to the vertices. This mechanism is required becuase vertex
    // buffers may be in device memory.
    VOID* pVertices;
    if( FAILED( g_pVB->Lock( 0, sizeof(vertices), (void**)&pVertices, 0 ) ) )
		MessageBox(NULL, "Cant Lock BBox VBuffer", "CObject::CalcBBox()", MB_OK);

    memcpy( pVertices, vertices, sizeof(vertices) );
    g_pVB->Unlock();


}


void CXObject::DrawBBox()
{
//	LPD3DXLINE Line = NULL;
	D3DXVECTOR3 pVert[4] = 	{
		// In Min.Z Place Rect
		D3DXVECTOR3(BBox_Min.x,BBox_Min.y, BBox_Min.z), D3DXVECTOR3(BBox_Max.x,BBox_Min.y, BBox_Min.z),
		D3DXVECTOR3(BBox_Max.x,BBox_Max.y, BBox_Min.z), D3DXVECTOR3(BBox_Min.x,BBox_Max.y, BBox_Min.z)
	};



	g_pd3dDevice->SetStreamSource( 0, g_pVB, 0, sizeof(CUSTOMVERTEX) );
    g_pd3dDevice->SetFVF( D3DFVF_CUSTOMVERTEX );
    g_pd3dDevice->DrawPrimitive( D3DPT_TRIANGLEFAN, 0, 2 );
	g_pd3dDevice->DrawPrimitive( D3DPT_LINESTRIP, 0, 5 );


}



// DRAW MESH.
void CXObject::Draw(int mode)
{
	
	D3DXMATRIXA16 matTex0, matTex1;
	D3DXMATRIXA16 matModel, matMove, matRotateX, matRotateY, matRotateZ, matScale,
		addRotateX, addRotateY, addRotateZ;
	
	D3DXMatrixRotationX( &matRotateX, Angle.x / 57.2f);
	D3DXMatrixRotationY( &matRotateY, Angle.y / 57.2f);
	D3DXMatrixRotationZ( &matRotateZ, Angle.z / 57.2f);

    D3DXMatrixTranslation( &matMove, Position.x, Position.y, Position.z );
	
	/*matModel *= matRotateX;
	matModel *= matRotateY;
	matModel *= matRotateZ;*/
	/*D3DXMatrixMultiply(&matModel,&matModel,&matRotateX);
	D3DXMatrixMultiply(&matModel,&matModel,&matRotateY);*/

	D3DXMatrixScaling(&matScale, Scale.x, Scale.y, Scale.z);

	D3DXMatrixMultiply(&matModel,&matModel,&matScale);
	D3DXMatrixMultiply(&matModel,&matRotateY,&matRotateX);
	D3DXMatrixMultiply(&matModel,&matModel,&matRotateZ);
	D3DXMatrixMultiply(&matModel,&matModel,&matMove);
	//matModel = matRotateX * matRotateY * matRotateZ * matMove * extMatrix;/*addRotateX * addRotateY * addRotateZ*/;

	g_pd3dDevice->SetTransform( D3DTS_WORLD, &matModel );

	// Meshes are divided into subsets, one for each material. Render them in
    // a loop
    for( DWORD i=0; i<NumMaterials; i++ )
    {
		
		// Scale
		D3DXMatrixScaling( &matTex0, TextureScale[0].x, TextureScale[0].y, TextureScale[0].z);
		g_pd3dDevice->SetTransform( D3DTS_TEXTURE0, &matTex0 );

		// Set Mip-Map & Bilinear
		g_pd3dDevice->SetSamplerState( 0, D3DSAMP_MINFILTER, D3DTEXF_LINEAR );
	    g_pd3dDevice->SetSamplerState( 0, D3DSAMP_MAGFILTER, D3DTEXF_LINEAR );
		g_pd3dDevice->SetSamplerState( 0, D3DSAMP_MIPFILTER, D3DTEXF_LINEAR );//D3DTEXF_NONE );
		

		// Set the material and texture for this subset
        g_pd3dDevice->SetMaterial( &MeshMaterials[i] );

        
		// Set UW For First Tex
		if (TextureMode[1] == PL_TEX_NORMAL) g_pd3dDevice->SetTextureStageState( 0, D3DTSS_TEXTURETRANSFORMFLAGS, D3DTTFF_COUNT2 );
		else g_pd3dDevice->SetTextureStageState( 0, D3DTSS_TEXTURETRANSFORMFLAGS, D3DTTFF_COUNT3 );
		g_pd3dDevice->SetTextureStageState( 0, D3DTSS_TEXCOORDINDEX, TextureMode[0] );

		// Set First Texture
		g_pd3dDevice->SetTexture( 0, MeshTextures[i] );
		if (bMultiTexture){
			g_pd3dDevice->SetTextureStageState( 0, D3DTSS_COLOROP,   D3DTOP_MODULATE);
			g_pd3dDevice->SetTextureStageState( 0, D3DTSS_COLORARG1, D3DTA_TEXTURE );
	        g_pd3dDevice->SetTextureStageState( 0, D3DTSS_COLORARG2, D3DTA_DIFFUSE );
		    g_pd3dDevice->SetTextureStageState( 0, D3DTSS_ALPHAOP,   D3DTOP_DISABLE );


		}else g_pd3dDevice->SetTextureStageState( 0, D3DTSS_COLOROP,   D3DTOP_MODULATE);
			//g_pd3dDevice->SetTextureStageState( 0, D3DTSS_COLOROP,   D3DTOP_ADDSIGNED );
			

		// Set Second Texture
		if (bMultiTexture){
			// Set Mip-Map & Bilinear For Second Tex
			/*g_pd3dDevice->SetSamplerState( 1, D3DSAMP_MINFILTER, D3DTEXF_LINEAR );
			g_pd3dDevice->SetSamplerState( 1, D3DSAMP_MAGFILTER, D3DTEXF_LINEAR );
			g_pd3dDevice->SetSamplerState( 1, D3DSAMP_MIPFILTER, D3DTEXF_NONE );
			
			// Set UW For Second Tex
			g_pd3dDevice->SetTextureStageState( 1, D3DTSS_TEXCOORDINDEX , /*D3DTSS_TCI_SPHEREMAP*/
			/*D3DTSS_TCI_CAMERASPACEREFLECTIONVECTOR);
			
			// Set Texture
			g_pd3dDevice->SetTexture( 1, SecondTexture );
			g_pd3dDevice->SetTextureStageState( 1, D3DTSS_COLOROP,   D3DTOP_ADD ); // D3DTOP_MODULATE4X );
        */
			// Scale
			D3DXMatrixScaling( &matTex1, TextureScale[1].x, TextureScale[1].y, TextureScale[1].z);
			g_pd3dDevice->SetTransform( D3DTS_TEXTURE1, &matTex1 );

			// Set Mip-Map & Bilinear For Second Tex
			g_pd3dDevice->SetSamplerState( 1, D3DSAMP_MINFILTER, D3DTEXF_LINEAR );
			g_pd3dDevice->SetSamplerState( 1, D3DSAMP_MAGFILTER, D3DTEXF_LINEAR );
			g_pd3dDevice->SetSamplerState( 1, D3DSAMP_MIPFILTER, D3DTEXF_LINEAR );//D3DTEXF_NONE );
			
			// Set UW For Second Tex
			if (TextureMode[1] == PL_TEX_NORMAL) g_pd3dDevice->SetTextureStageState( 1, D3DTSS_TEXTURETRANSFORMFLAGS, D3DTTFF_COUNT2 );
			else g_pd3dDevice->SetTextureStageState( 1, D3DTSS_TEXTURETRANSFORMFLAGS, D3DTTFF_COUNT3 );
			g_pd3dDevice->SetTextureStageState( 1, D3DTSS_TEXCOORDINDEX , TextureMode[1]);//D3DTSS_TCI_CAMERASPACEREFLECTIONVECTOR);
			
			// Set Texture
			g_pd3dDevice->SetTexture( 1, SecondTexture );
			g_pd3dDevice->SetTextureStageState( 1, D3DTSS_COLOROP,   TextureAdd[1]);//D3DTOP_ADD );  //D3DTOP_MODULATE4X );
        
		}


        // Draw the mesh subset
        Mesh->DrawSubset( i );
	
    }

	// Turn Off MultiTexturing
	g_pd3dDevice->SetTextureStageState( 1, D3DTSS_COLOROP,  D3DTOP_DISABLE );
	//g_pd3dDevice->SetTextureStageState( 0, D3DTSS_COLOROP,   D3DTOP_MODULATE);
	//DrawBBox();

}


// DRAW MESH.
/*void CXObject::Draw(int mode)
{
			LPDIRECT3DVERTEXBUFFER9 pVB;
            LPDIRECT3DINDEXBUFFER9 pIB;
			

            Mesh->GetVertexBuffer( &pVB );
            Mesh->GetIndexBuffer( &pIB );

            g_pd3dDevice->SetFVF( Mesh->GetFVF() );

            g_pd3dDevice->SetStreamSource( 0, pVB, 0, sizeof(D3DVERTEX) );
        
			g_pd3dDevice->SetIndices( pIB );

			 g_pd3dDevice->DrawIndexedPrimitive( D3DPT_TRIANGLELIST, 0, 
                    0, Mesh->GetNumVertices(),
                    0, Mesh->GetNumFaces() );

			 pVB->Release();
             pIB->Release();

}
*/

// LOAD MESH.
HRESULT CXObject::Load(char* filename)
{
	char* filename_bk = filename;
	Visible = TRUE;

	Mesh          = NULL; // Our mesh object in sysmem
	MeshMaterials = NULL; // Materials for our mesh
	MeshTextures  = NULL; // Textures for our mesh
	NumMaterials = 0L;   // Number of mesh materials


	LPD3DXBUFFER pD3DXMtrlBuffer;

	// Make filename in topest directory
	char strtemp[64];
	strcpy(strtemp, "..\\");
	strcpy(strtemp+3, filename);
	//MessageBox(NULL, strtemp, "X Load", MB_OK);
	//MessageBox(NULL, filename, "X Load", MB_OK);
    
	// Load the mesh from the specified file
    if( FAILED( D3DXLoadMeshFromX( filename, D3DXMESH_SYSTEMMEM, 
                                   g_pd3dDevice, NULL, 
                                   &pD3DXMtrlBuffer, NULL, &NumMaterials, 
                                   &Mesh ) ) )
    {
        // If model is not in current folder, try parent folder
        if( FAILED( D3DXLoadMeshFromX( strtemp, D3DXMESH_SYSTEMMEM, 
                                    g_pd3dDevice, NULL, 
                                    &pD3DXMtrlBuffer, NULL, &NumMaterials, 
                                    &Mesh ) ) )
        {
            MessageBox(NULL, "Could not find X file", "Load XMesh", MB_OK);
            return E_FAIL;
        }
    };

	// Set Vert Format
	//Mesh.SetFVF( m_pd3dDevice, D3DVERTEX::FVF );
	
	char *s_tmp="", s_pref[128]="" ;
	int slen1, slen2;
	//strcpy(s_pref,"/0");
	//s_pref[1] = "123";
	if (s_tmp = strchr(filename,'/')) {
		//MessageBox(NULL, s_tmp, "!!!X Load ?", MB_OK);
		slen1 = strlen(filename);
		slen2 = strlen(s_tmp);
		strncpy(s_pref, filename, (slen1-slen2+1));
		//lstrcpyn(s_pref,filename,(slen1-slen2+1));
		//MessageBox(NULL, s_pref, "!!!X TexPref", MB_OK);
		strcpy(Path,s_pref);
		//MessageBox(NULL, Path, "!!!X TexPref", MB_OK);
	}

	
    // We need to extract the material properties and texture names from the 
    // pD3DXMtrlBuffer
    D3DXMATERIAL* d3dxMaterials = (D3DXMATERIAL*)pD3DXMtrlBuffer->GetBufferPointer();
    MeshMaterials = new D3DMATERIAL9[NumMaterials];
    MeshTextures  = new LPDIRECT3DTEXTURE9[NumMaterials];

    for( DWORD i=0; i<NumMaterials; i++ )
    {
        // *** TEXTURES *** //
		/*char tex_real_fname[128], ch1_tmp[128];
		char *s_tmp;
		if (s_tmp = strrchr(d3dxMaterials[i].pTextureFilename,'\\')) {
		MessageBox(NULL, s_tmp, "!!!X Load Tex", MB_OK);
			//s_tmp = 
			strcpy(ch1_tmp, s_tmp);
			sprintf(ch1_tmp,"%s\n",ch1_tmp);
			MessageBox(NULL, ch1_tmp, "!!!X Load Tex", MB_OK);
		}*/
		
		
		// Copy the material
        MeshMaterials[i] = d3dxMaterials[i].MatD3D;

        // Set the ambient color for the material (D3DX does not do this)
        MeshMaterials[i].Ambient = MeshMaterials[i].Diffuse;

        MeshTextures[i] = NULL;
        if( d3dxMaterials[i].pTextureFilename != NULL && 
            lstrlen(d3dxMaterials[i].pTextureFilename) > 0 )
        {
            // Create the texture
            if( FAILED( D3DXCreateTextureFromFile( g_pd3dDevice, 
                                                d3dxMaterials[i].pTextureFilename, 
                                                &MeshTextures[i] ) ) )
            {
				//MessageBox(NULL, d3dxMaterials[i].pTextureFilename, "warn!!! p1", MB_OK);
                //NOT->// If texture is not in current folder, try parent folder
				// Try in same foler as a .X file
                const TCHAR* strPrefix = s_pref;//TEXT("..\\");
                const int lenPrefix = lstrlen( strPrefix );
                TCHAR strTexture[MAX_PATH];
                lstrcpyn( strTexture, strPrefix, MAX_PATH );
                lstrcpyn( strTexture + lenPrefix, d3dxMaterials[i].pTextureFilename, MAX_PATH - lenPrefix );
                //MessageBox(NULL, strTexture, "warn!!! p2", MB_OK);
				// If texture is not in current folder, try parent folder
                if( FAILED( D3DXCreateTextureFromFile( g_pd3dDevice, 
                                                    strTexture, 
                                                    &MeshTextures[i] ) ) )
                {
                    MessageBox(NULL, "Could not find texture map", "MeshTex Load", MB_OK);
                }
            }
        }
    }

    // Done with the material buffer
    pD3DXMtrlBuffer->Release();

	// Calculate Bounding Box
	CalcBBox();

	return S_OK;
}




void CXObject::LoadSecondTexture(char* filename)
{
	bMultiTexture = TRUE;
	TextureMode[1] = D3DTSS_TCI_CAMERASPACEREFLECTIONVECTOR;
	// Create Second Texture
            if( FAILED( D3DXCreateTextureFromFile( g_pd3dDevice, 
                                                filename, 
                                                &SecondTexture ) ) )
            {
                // If texture is not in current folder, try parent folder
                const TCHAR* strPrefix = Path;//s_pref;//TEXT("..\\");
				//MessageBox(NULL, Path, "!!!X SEC Tex", MB_OK);
                const int lenPrefix = lstrlen( strPrefix );
                TCHAR strTexture[MAX_PATH];
                lstrcpyn( strTexture, strPrefix, MAX_PATH );
                lstrcpyn( strTexture + lenPrefix, filename, MAX_PATH - lenPrefix );
                // If texture is not in current folder, try parent folder
                if( FAILED( D3DXCreateTextureFromFile( g_pd3dDevice, 
                                                    strTexture, 
                                                    &SecondTexture ) ) )
                {
                    MessageBox(NULL, "Could not find second texture map", "Meshes.exe", MB_OK);
					bMultiTexture = FALSE;
                }
			}
}


void CXObject::CleanUp()
{
	for( DWORD i=0; i<NumMaterials; i++ ) {
		SAFE_RELEASE(Mesh);
		SAFE_RELEASE(MeshTextures[i]);//Materials[i]);
		SAFE_RELEASE(SecondTexture);
	}


}