#include <core.h>
#include <object.h>
#include <level.h>

#include <hge.h>
#include <hgeresource.h>
#include <hgesprite.h>
#include <hgefont.h>
#include <hgeparticle.h>
#include <stdio.h>

CCore *CCore::Engine = 0;

CCore::CCore(void) {
	Timer = 0.f;
	hge = 0;
	Engine = this;
	GAME_UPDATE_STEP_TIME = 1.0f / 60.0f;
	ShowDebug = true;
	mouseJoint = NULL;
}

CCore::~CCore(void) {
	delete fnt;
	if(hge) {
		hge->System_Shutdown();
		hge->Release();
	}
	Engine = 0;
	delete myWorld;
	myWorld = NULL;
}

bool CCore::Init(void) {
	hge = hgeCreate(HGE_VERSION);
	if(!hge) return false;

	hge->System_SetState(HGE_SHOWSPLASH,true);
	hge->System_SetState(HGE_INIFILE,"config.ini");
	hge->System_SetState(HGE_LOGFILE, "project.log");
	hge->System_SetState(HGE_FRAMEFUNC, CCore::StaticUpdate);
	hge->System_SetState(HGE_RENDERFUNC, CCore::StaticRender);
	hge->System_SetState(HGE_TITLE, "develop by igdc #48 (my first step C++/HGE/Box2D)");
	hge->System_SetState(HGE_FPS, 60);
	hge->System_SetState(HGE_WINDOWED, true);
	hge->System_SetState(HGE_SCREENWIDTH, 800);
	hge->System_SetState(HGE_SCREENHEIGHT, 600);
	hge->System_SetState(HGE_SCREENBPP, 32);
	hge->System_SetState(HGE_HIDEMOUSE, false);

	//hge->System_SetState(HGE_USERDATA, this);

	Width = hge->System_GetState(HGE_SCREENWIDTH);
	Height = hge->System_GetState(HGE_SCREENHEIGHT);
	halfWidth = Width / 2;
	halfHeight = Height / 2;

	worldAABB.lowerBound.Set(-50.0f, -50.0f);
	worldAABB.upperBound.Set(100.0f, 100.0f);
	b2Vec2 gravity(0.0f, 0.0f);
	myWorld = new b2World(worldAABB, gravity, true);  //bool doSleep = true;
	iterations = 10;

	// We need to clean the current flags
	flagsDraw = HGEDebugDraw::e_shapeBit|HGEDebugDraw::e_jointBit|HGEDebugDraw::e_jointBit|HGEDebugDraw::e_aabbBit|HGEDebugDraw::e_obbBit;
	DebugDraw.SetFlags(0);
	DebugDraw.SetHGE(hge);

	m_pointCount = 0;
	m_destructionListener.core = this;
	m_boundaryListener.core = this;
	m_contactListener.core = this;
	myWorld->SetDestructionListener(&m_destructionListener);
	myWorld->SetBoundaryListener(&m_boundaryListener);
	myWorld->SetContactListener(&m_contactListener);
	myWorld->SetDebugDraw(&DebugDraw);

	if(hge->System_Initiate()) {
		fnt=new hgeFont("data/fnt/font1.fnt");
		Level = new CLevel(Engine,hge,myWorld);
		Level->Load();
		GAME_UPDATE_OLD_TIME = hge->Timer_GetTime();

		hge->System_Start();

		Level->DeleteAllObject();
		Level=0;
	}
	return true;
}

bool CCore::Update(float delta) {
	// Process keys
	if (hge->Input_GetKeyState(HGEK_ESCAPE)) return true;
	myWorld->Step(GAME_UPDATE_STEP_TIME, iterations);
	Level->Update(GAME_UPDATE_STEP_TIME);

	for (int i = 0; i < Engine->m_pointCount; ++i)
	{
		m_points[i].state = e_contactRemoved;
	}
	m_pointCount = 0;

	if(hge->Input_KeyUp(HGEK_F5)) hge->System_Snapshot();
	return false;
}

bool CCore::Render(void) {
	hge->Gfx_BeginScene();
	hge->Gfx_Clear(0);
	Level->Render();

	fnt->printf(15, 15, HGETEXT_LEFT, "fps %d", hge->Timer_GetFPS());

	if (ShowDebug)
	{
		hge->Gfx_SetTransform(0,0,0,0,0,GAME_GLOBAL_SIZE,GAME_GLOBAL_SIZE);
		DebugDraw.SetFlags(flagsDraw);
		myWorld->Step(0,0);
		DebugDraw.SetFlags(0);
		hge->Gfx_SetTransform();
	}
	hge->Gfx_EndScene();
	return false;
}

bool CCore::Exit(void) {
	return true;
}

bool CCore::StaticUpdate(void) {
	return Engine->Update(Engine->hge->Timer_GetDelta());
}

bool CCore::StaticRender(void) {
	return Engine->Render();
}

//***b2World***\\

void DestructionListener::SayGoodbye(b2Joint* joint)
{
	if (core->mouseJoint == joint)
	{
		core->mouseJoint = NULL;
	}
	else
	{
		core->JointDestroyed(joint);
	}
}

void BoundaryListener::Violation(b2Body* body)
{
	/*if (core->m_bomb != body)
	{
		core->BoundaryViolated(body);
	}*/
}

void ContactListener::Add(const b2ContactPoint* point)
{
	if (core->m_pointCount == k_maxContactPoints)
	{
		return;
	}

	ContactPoint* cp = core->m_points + core->m_pointCount;
	cp->shape1 = point->shape1;
	cp->shape2 = point->shape2;
	cp->position = point->position;
	cp->normal = point->normal;
	cp->id = point->id;
	cp->state = e_contactAdded;

	++core->m_pointCount;
}

void ContactListener::Persist(const b2ContactPoint* point)
{
	if (core->m_pointCount == k_maxContactPoints)
	{
		return;
	}

	ContactPoint* cp = core->m_points + core->m_pointCount;
	cp->shape1 = point->shape1;
	cp->shape2 = point->shape2;
	cp->position = point->position;
	cp->normal = point->normal;
	cp->id = point->id;
	cp->state = e_contactPersisted;

	++core->m_pointCount;
}

void ContactListener::Remove(const b2ContactPoint* point)
{
	/*if (core->m_pointCount == k_maxContactPoints)
	{
		return;
	}

	ContactPoint* cp = core->m_points + core->m_pointCount;
	cp->shape1 = point->shape1;
	cp->shape2 = point->shape2;
	cp->position = point->position;
	cp->normal = point->normal;
	cp->id = point->id;
	cp->state = e_contactRemoved;

	++core->m_pointCount;
	int i=0;
	while (i < core->m_pointCount)
	{
		if (core->m_points->id.key == point->id.key)
		{
			core->m_points->state = e_contactRemoved;
		}
		i++;
	}
	--core->m_pointCount;*/
}